-- modificated 2010-2013 Matthias Richter's Camera.lua module
-- * Modified

local cos, sin, gr = math.cos, math.sin, love.graphics

local camera	= {}
camera.__index	= camera

function camera.new( x, y, zoom, rot, keys )	--*
    x,y  = x or gr.getWidth()/2, y or gr.getHeight()/2
    zoom = zoom or 0.8
    rot  = rot or 0
	keys = keys or {up={'up','w'},down={'down','s'},left = {'left','a'}, right={'right','d'}}
    return setmetatable( {x = x, y = y, scale = zoom, rot = rot,keys = keys}, camera )
	end

function camera:setKeys( keys )	-- provides for overriding default key settings *
	self.keys = keys
	end

function camera:setLocation( x, y ) self.x, self.y = x, y end	--*	formerly lookAt
function camera:getLocation( x, y ) return gr.getWidth()/2,gr. getHeight()/2 end	--*

function camera:move( dx, dy ) self.x, self.y = self.x + dx, self.y + dy end
function camera:moveUnrotated(x,y,rot)		-- move relative to screen, not rotated camera *
	local c, s = cos(-rot), sin(-rot)
	self.x, self.y = self.x + c*x - s*y, self.y + s*x + c*y
	end

function camera:update()		--*
    if love.keyboard.isDown( 'o' ) then
        self:zoom( 0.99 )
		elseif love.keyboard.isDown( 'i' ) then
        self:zoom( 1.01 )
		end
    if love.keyboard.isDown( unpack(self.keys.left) ) then
        self:moveUnrotated( -2, 0, rotate.angle )	-- Note use of moveUnrotated!
		elseif love.keyboard.isDown( unpack(self.keys.right) ) then
        self:moveUnrotated( 2, 0, rotate.angle )
		end
    if love.keyboard.isDown( unpack(self.keys.up) ) then
        self:moveUnrotated( 0, -2, rotate.angle )
		elseif love.keyboard.isDown( unpack(self.keys.down) ) then
        self:moveUnrotated( 0, 2, rotate.angle )
		end
	end

function camera:pos() return self.x, self.y end
function camera:mousepos() return self:worldCoords(love.mouse.getPosition()) end

-- rotates around center of screen ( not graph )
function camera:rotate( rad ) self.rot = self.rot + rad end
function camera:rotateTo( rad ) self.rot = rad end
function camera:getRotation( ) return self.rot end	--*

function camera:zoom( mul ) self.scale = self.scale * mul end
function camera:getZoom() return self.scale end
function camera:zoomTo( zoom ) self.scale = zoom end

function camera:attach()
    local cx, cy = gr.getWidth()/(2*self.scale), gr.getHeight()/(2*self.scale)
    gr.push()
	gr.scale( self.scale )
	gr.translate( cx, cy )
	gr.rotate( self.rot )
	gr.translate( -self.x, -self.y )
	end

function camera:detach() gr.pop() end

function camera:draw( func )
    self:attach()
    func()
    self:detach()
	end

function camera:cameraCoords( x, y )		-- unused
    local w, h = gr.getWidth(), gr.getHeight()
    local c, s = cos( self.rot ), sin( self.rot )
    local x, y = x - self.x, y - self.y
    local x, y = c * x - s * y, s * x + c * y
    return x * self.scale + w/2, y * self.scale + h/2
	end

function camera:worldCoords( x, y )		-- used by mousepos
    local w, h = gr.getWidth(), gr.getHeight()
    local c, s = cos(-self.rot), sin(-self.rot)
    local x, y = (x - w/2) / self.scale, (y - h/2) / self.scale
    x, y = c*x - s*y, s*x + c*y
    return x+self.x, y+self.y
	end

return camera
