local min, max, sqrt = math.min, math.max, math.sqrt

-- @param id - A unique id used to reference node
-- @param x  - Node x spawn location
-- @param y  - Node y spawn location
-- @param anchor - Node location fixed or not

local Node = {
	clamp = function( mn, val, mx )	-- prevents graph 'exploding'
        return math.max( mn, min( val, mx ) )
		end
		}

function Node.new( id, x, y,  anchor,color, size, mass )
    local tab = {
	id	= id,
    x	= x or 0,
	y 	= y or 0,			-- current node position
    ax	= 0,
	ay	= 0,				-- acceleration
    vx	= 0,
	vy	= 0,				-- velocity
	mass= mass or param.mass,
	size= size or sprite.size,
	anchor	= anchor or false,
	color = color or { 130, 190, 238 },
	applyForce  = function( self, force )	-- calculate node acceleration
        self.ax = Node.clamp( -force.max, self.ax + force.x, force.max )
        self.ay = Node.clamp( -force.max, self.ay + force.y, force.max )
		end,

	getDist = function( self, tx, ty )		-- node sepatation - node(x,y) and target(tx,ty)
		local dx, dy = self.x-tx, self.y-ty
		return sqrt( dx * dx + dy * dy ) + 0.1
		end,

    attractTo=function( self, node )		-- inter-nodal attraction
		local tx, ty	= node.x, node.y
        local distance	= self:getDist( tx, ty )
        local dx, dy	= ( self.x-tx ) / distance, ( self.y-ty ) / distance
        local strength	= -1 * force.spring * distance * 0.5
		force.x, force.y = dx * strength, dy * strength
		self:applyForce( force )
		end,

    repelFrom= function( self, node )	-- node repulsion
		local tx, ty	= node.x, node.y
        local distance	= self:getDist( tx, ty )
        local dx, dy	= ( self.x-tx ) / distance, ( self.y-ty ) / distance
        local strength	= force.change * (( param.mass * self.mass ) / ( distance * distance ))
        force.x, force.y = dx * strength, dy * strength
		self:applyForce( force )
		end,

    move = function( self, dt )		-- update node's position (vel, acc and damp)
        self.vx		= (self.vx + self.ax * dt * param.speed) * param.damping
        self.vy		= (self.vy + self.ay * dt * param.speed) * param.damping
        self.x, self.y	= self.x + self.vx, self.y + self.vy	-- x, y is the actual location of node
        self.ax, self.ay	= 0, 0
		end,
	}

	return tab

	end

return Node
