-- Demo refractured by Ref
local Camera	= require 'CameraX'
local graph		= require 'Graphoon.Graph'
local menu		= require 'menu'
local text		= require 'text'
local pulse		= require( 'pulse' )
color			= require( 'color' )
local gr		= love.graphics

local grabbed, selected, nodeIDs, camera
local floor,  pi, sin, cos = math.floor, math.pi, math.sin, math.cos

sprite	= {
	image			= gr.newImage( 'file.png' ), size = 18 }
	sprite.ht		= sprite.image:getHeight()
	sprite.scale	= sprite.size/sprite.ht
	sprite.offset	= sprite.ht / 2

anchor = {
	image			= gr.newImage( 'anchor.png'), scale = 0.5 }
	anchor.w		= anchor.image:getWidth()
	anchor.h		= anchor.image:getHeight()
icon				= gr.newImage( 'icon.png' )

-- lower 'change' or higher 'spring' reduces edge lengths
force			= { spring = 0.005, change  = 20,	max  = 4 }
param			= { speed  = 300,   damping = 0.9,	mass = 3 }
rotate			= { angle = 0 }
fontsize		= sprite.size

active 			= {				-- key codes for enabling/disabling features
					l	= true,	-- labels
					e	= true,	-- edges
					p	= true,	-- points (nodes)
					c	= false,-- calls
					r	= false,-- rotation
					b	= false,-- boundingbox
					z	= false,-- stats
					x	= false,-- graph center
					m	= true, -- controls
					u	= false,-- save graph
					t	= false,-- change label
					d	= false,-- node data
					a	= false,-- toggle anchor
					f	= false,-- increment mass
					g	= false,-- decrement mass
					q	= false,-- increment size
					w	= false,-- decrement size
					k	= false,-- log key strokes
					f1	= false,-- inc spring
					f2	= false,-- dec spring
					f3	= false,-- inc change
					f4	= false,-- dec change
					s	= false,-- shuttle
					}

local data		= { sets = { 'ethanol','loveRun', 'GraphoonData', 'EvilDead' }, selected = 2 }
local screen	= { wd = 1200, ht = 700 }
keylog	= {}
shuttle		= {		-- orbiting node
	x		= 100,
	y		= 150,
	rx		= 300,	-- radius
	ry		= 100,
	angle	= 0,
	size	= 30,
	mass	= 20,
	color	= {0,255,0},
	cx		= screen.wd/2,	-- center
	cy		= screen.ht/2,
	anchor	= false,
	id		= 'shuttle',
	rot 	= function(self,tk)
				self.angle = self.angle + tk
				graph.nodes[self.id].x=self.rx*cos(self.angle)+self.cx
				graph.nodes[self.id].y=self.ry*sin(self.angle)+self.cy
				end,
	init	= function(self)
				graph:addNode(self.id,self.x,self.y,self.anchor,self.color,self.size,self.mass)
				end
			}

function love.load()
    gr.setBackgroundColor( color.background )
	love.window.setMode( screen.wd, screen.ht, {borderless = true } )
    gr.setFont( gr.newFont( fontsize ) )
	graph	= graph.new( data.sets[ data.selected ] )
    nodeIDs = 0				-- id for nodes added later
	keys	= { up = {'up'}, down = {'down'}, left = {'left'}, right = {'right'} }
    camera	= Camera.new()	-- for zoom and graph movement
	camera:setKeys( keys )	-- overrides default control key settings in camera
	menu:create()			-- keyboard control keys - create a canvas
	end

function love.update( dt )
    graph:update( dt )
    if grabbed then grabbed.x, grabbed.y = camera:mousepos() end
	if active.r then
		rotate.angle = rotate.angle + dt/10
		camera:rotateTo( rotate.angle )
		end
	menu:update( dt )
	if not active.t then camera:update() end	-- active.t blocks camera keys
	if active.u then graph.save( 'Graphoon' ) active.u = false end
	if grabbed and active.a then
		grabbed.anchor	= not grabbed.anchor
		active.a = false
		end
	if grabbed and active.f then
		grabbed.mass = grabbed.mass+1
		active.f = false
		end
	if grabbed and active.g then
		grabbed.mass = grabbed.mass-1
		active.g = false
		end
	if grabbed and active.q then
		grabbed.size = grabbed.size+1
		active.q = false
		end
	if grabbed and active.w then
		grabbed.size	= grabbed.size-1
		active.w		= false
		end
	if grabbed then pulse:update( dt ) end
	if shuttle.running then shuttle:rot(dt) end
	end

function love.draw()
	if active.m then menu:draw( 1030, 275 ) end		-- list of key control options
	if active.k then graph:showKeylog() end			-- display key strokes
    camera:draw(
		function() graph:draw()
        if selected then
            local mx, my = camera:mousepos()
            gr.line( selected.x, selected.y, mx, my )
            gr.setColor( color.cursor )
			gr.circle( 'line', mx, my , 30 )
			end
		if active.b then graph.showBoundingBox() end	-- show bounding box
		if grabbed then									-- show which node grabbed
			gr.setColor(color.grabbed[1],color.grabbed[2],color.grabbed[3], pulse.alpha )
			gr.circle( 'line', grabbed.x, grabbed.y, grabbed.size/2, 20 )
			gr.draw(anchor.image,grabbed.x,grabbed.y,-rotate.angle-pi/4,anchor.scale,anchor.scale,anchor.w/2,anchor.h/2)
			end
		end
		)
	if active.z then graph.showStats( 20, 495 ) end	-- show performance info
	if active.x then graph.showOrientation(camera:getRotation(),camera:getLocation() ) end
	if active.t then text:print( 300, screen.ht-30 )	end	-- show text as typed
	if active.d and grabbed then graph:nodeData(grabbed) else	active.d  = false end
	if active.f1 then force.spring = force.spring + .001		active.f1 = false end
	if active.f2 then force.spring = force.spring - .001		active.f2 = false end
	if active.f3 then force.change = force.change + 5			active.f3 = false end
	if active.f4 then force.change = force.change - 5			active.f4 = false end
	if active.s  then
		if not shuttle.running then
			shuttle:init()
			shuttle.running = true
			active.s = false
			else
			shuttle.running=false
			active.s=false
			graph:removeNode( graph.nodes[shuttle.id] )
			end
		end
	end

function love.mousepressed( _, _, button )	-- grab & connect nodes
    local mx, my	= camera:mousepos()
    local node		= graph:getNodeAt( mx, my, 30 )-- 30 = how close to node
	if node and not grabbed then
		if button == 1 and not selected then
			grabbed = node
			if active.k then
				if text.ready then keylog[#keylog+1] = ( 'Node %s replaced by "%s"' ):format( grabbed.id, text.new )
				else
				keylog[#keylog+1] = ( 'Button %s pressed - node: "%s" grabbed' ):format( button, node.id )
				end
				end
			if text.ready and #text.new ~= 0 then
				local new = text.new
				oldID = text:replace( node )
				end
			elseif button == 1 and selected then
				graph:addEdge( selected, node )
				if active.k then keylog[#keylog+1] = ( 'Button %s pressed - nodes "%s" and "%s" attached' ):format( button, node.id, selected.id ) end
				selected = nil
			elseif button == 2 and not selected then
				selected = node
				if active.k then keylog[#keylog+1] = ( 'Button %s pressed - node "%s" selected' ):format( button, node.id ) end
			end
		else
			if active.k and grabbed then
				keylog[#keylog+1] = ( 'Button %s pressed - node "%s released' ):format( button, grabbed.id )
				else
				keylog[#keylog+1] = ( 'Button %s pressed' ):format( button )
				end
			selected, grabbed = nil, nil
		end
	end

function love.keypressed( key )
	if active.t then
		text:enter( key )
		if text.ready and active.k then keylog[#keylog+1]= ( 'Key %s pressed - new label name is "%s" '):format( key, text.new ) end
		else
		if key == 'space' and grabbed then
			local newNode = graph:addNode( nodeIDs, camera:mousepos() )
			graph:addEdge( grabbed, newNode )
			if active.k then
				keylog[#keylog+1] = ( 'Key %s pressed - new Node "%s" created' ):format( key, nodeIDs )
				end
			nodeIDs = nodeIDs + 1
			elseif key == 'backspace' and grabbed then
				graph:removeNode( grabbed )
				graph:removeEdges( grabbed )
				if active.k then
					keylog[#keylog+1] = ( 'Key %s pressed - Node "%s" removed' ):format( key, grabbed.id )
					end
				grabbed = nil
			elseif key == 'n' then
				data.selected = data.selected < #data.sets and data.selected + 1 or 1
				graph = graph.new( data.sets[data.selected] )
				shuttle:init()
				if active.k then
					keylog[#keylog+1] = ( 'Key %s pressed - data set "%s" selected' ):format( key, data.sets[data.selected] )
					end
			elseif pressed( key, active) then
				active[key] = not active[key]
				if active.k then
					keylog[#keylog+1] = ( 'Key "%s" pressed' ):format( key )
					end
			elseif key == 'escape'	then love.event.quit()
			end
		end
	end

function pressed( k, list )	-- determines if index k is present in table
	for s,_ in pairs( list ) do if s == k then return true end end
	return false
	end
